from distutils.core import setup
setup(
  name = 'pyGT511C3',
  packages = ['pyGT511C3'], # this must be the same as the name above
  version = '0.1b',
  description = 'Quick Corp Python driver for Fingerprint Scanner - TTL (GT-511C3)',
  author = 'Jean Machuca',
  author_email = 'correojean@gmail.com',
  url = 'https://github.com/QuickGroup/pyGT511C3', # use the URL to the github repo
  download_url = 'https://github.com/QuickGroup/pyGT511C3/legacy.tar.gz/0.0.1', # I'll explain this in a second
  keywords = ['pyGT511C3', 'GT511C3', 'fingerprint','scanner','ttl','image','finger'], # arbitrary keywords
  classifiers = [],
)
